@@ -66,30 +66,21 @@ def factoring(
 
 def all_possible_factors(
     given_clause: grammar.Clause,
-    label_prefix: str,
-    starting_label_index: int,
 ) -> List[grammar.Clause]:
     """
     one of the four basic building blocks of the Given Clause algorithm
 
-    >>> from gym_saturation.grammar import Predicate, Variable, Function
-    >>> all_possible_factors(grammar.Clause([grammar.Literal(False, Predicate("this_is_a_test_case", []))]), "inferred_", 0)
-    Traceback (most recent call last):
-     ...
-    ValueError: no label: Clause(literals=[Literal(negated=False, atom=Predicate(name='this_is_a_test_case', arguments=[]))], label=None, inference_parents=None, inference_rule=None, processed=None, birth_step=None)
-    >>> all_possible_factors(grammar.Clause([grammar.Literal(False, Predicate("p", [Function("f", [])])), grammar.Literal(False, Predicate("p", [Variable("X")])), grammar.Literal(False, Predicate("q", []))], label="this_is_a_test_case"), "inferred_", 0)
-    [Clause(literals=[Literal(negated=False, atom=Predicate(name='q', arguments=[])), Literal(negated=False, atom=Predicate(name='p', arguments=[Function(name='f', arguments=[])]))], label='inferred_0', inference_parents=['this_is_a_test_case'], inference_rule='factoring', processed=None, birth_step=None)]
+    >>> from gym_saturation.parsing.tptp_parser import TPTPParser
+    >>> parser = TPTPParser()
+    >>> clause = parser.parse("cnf(one, axiom, p(c) | p(X) | q).", "")[0]
+    >>> all_possible_factors(clause)  # doctest: +ELLIPSIS
+    [cnf(..., hypothesis, q() | p(c), inference(factoring, [], [one])).]
 
     :param given_clause: a new clause which should be combined with all the
         processed ones
-    :param label_prefix: generated clauses will be labeled with this prefix
-    :param starting_label_index: generated clauses will be indexed starting
-        with this number
     :returns: results of all possible factors with each one from
         ``clauses`` and the ``given_clause``
     """
-    if given_clause.label is None:
-        raise ValueError(f"no label: {given_clause}")
     factors: List[grammar.Clause] = []
     for i, literal_one in enumerate(given_clause.literals):
         for j in range(i + 1, len(given_clause.literals)):
@@ -115,7 +106,6 @@ def all_possible_factors(
             literals=factor.literals,
             inference_parents=[given_clause.label],
             inference_rule="factoring",
-            label=label_prefix + str(starting_label_index + ord_num),
         )
         for ord_num, factor in enumerate(factors)
     ]
