@@ -54,59 +54,50 @@ def reflexivity_resolution(
 
 def all_possible_reflexivity_resolvents(
     given_clause: grammar.Clause,
-    label_prefix: str,
-    label_index_base: int,
 ) -> List[grammar.Clause]:
     """
     one of the four basic building blocks of the Given Clause algorithm
 
-    >>> from gym_saturation.grammar import Predicate, Variable, Function
-    >>> all_possible_reflexivity_resolvents(grammar.Clause([grammar.Literal(False, Predicate("this_is_a_test_case", []))]), "inferred_", 1)
-    Traceback (most recent call last):
-     ...
-    ValueError: no label: Clause(literals=[Literal(negated=False, atom=Predicate(name='this_is_a_test_case', arguments=[]))], label=None, inference_parents=None, inference_rule=None, processed=None, birth_step=None)
-    >>> all_possible_reflexivity_resolvents(grammar.Clause([grammar.Literal(False, Predicate("p", [Variable("X")])), grammar.Literal(False, Predicate("=", [Variable("X"), Function("g", [])])), grammar.Literal(True, Predicate("!=", [Function("f", []), Function("g", [])]))], label="this_is_a_test_case"), "inferred_", 0)
-    [Clause(literals=[Literal(negated=False, atom=Predicate(name='p', arguments=[Function(name='g', arguments=[])])), Literal(negated=True, atom=Predicate(name='!=', arguments=[Function(name='f', arguments=[]), Function(name='g', arguments=[])]))], label='inferred_0', inference_parents=['this_is_a_test_case'], inference_rule='reflexivity_resolution', processed=None, birth_step=None)]
+    >>> from gym_saturation.parsing.tptp_parser import TPTPParser
+    >>> parser = TPTPParser()
+    >>> clause = parser.parse("cnf(this_is_a_test_case, axiom, p(X) | ~ X=a | b != a).", "")[0]
+    >>> all_possible_reflexivity_resolvents(clause)  # doctest: +ELLIPSIS
+    [cnf(..., hypothesis, p(a) | ~b = a, inference(reflexivity_resolution, [], [this_is_a_test_case])).]
 
     :param given_clause: a new clause which should be combined with all the
         processed ones
-    :param label_prefix: generated clauses will be labeled with this prefix
-    :param label_index_base: generated clauses will be indexed starting
-        with this number
     :returns: results of all possible reflexivity resolvents with each one from
         ``clauses`` and the ``given_clause``
     """
-    if given_clause.label is None:
-        raise ValueError(f"no label: {given_clause}")
     reflexivity_resolvents: List[grammar.Clause] = []
     for i, a_literal in enumerate(given_clause.literals):
         if (
-            not a_literal.negated
+            a_literal.negated
             and a_literal.atom.name == "="
-            or a_literal.negated
+            or not a_literal.negated
             and a_literal.atom.name == "!="
         ) and len(a_literal.atom.arguments) == 2:
             a_clause = grammar.Clause(
                 given_clause.literals[:i] + given_clause.literals[i + 1 :]
             )
-            try:
-                reflexivity_resolvents.append(
-                    reflexivity_resolution(
-                        a_clause,
-                        (
-                            a_literal.atom.arguments[0],
-                            a_literal.atom.arguments[1],
-                        ),
+            if a_clause.literals:
+                try:
+                    reflexivity_resolvents.append(
+                        reflexivity_resolution(
+                            a_clause,
+                            (
+                                a_literal.atom.arguments[0],
+                                a_literal.atom.arguments[1],
+                            ),
+                        )
                     )
-                )
-            except NonUnifiableError:
-                pass
+                except NonUnifiableError:
+                    pass
     return [
         grammar.Clause(
             literals=reflexivity_resolvent.literals,
             inference_parents=[given_clause.label],
             inference_rule="reflexivity_resolution",
-            label=label_prefix + str(label_index_base + ord_num),
         )
         for ord_num, reflexivity_resolvent in enumerate(reflexivity_resolvents)
     ]
