@@ -46,7 +46,7 @@ class CNFParser(Transformer):
     ...    cnf(test, axiom, f(X, g(Y), h(Z, c1)) = f(X, Y, c2)
     ...    | ~ better(f(X), g(Y)) | $false | this_is_a_test_case).
     ... '''))
-    Clause(literals=[Literal(negated=False, atom=Predicate(name='=', arguments=[Function(name='f', arguments=[Variable(name='X'), Function(name='g', arguments=[Variable(name='Y')]), Function(name='h', arguments=[Variable(name='Z'), Function(name='c1', arguments=[])])]), Function(name='f', arguments=[Variable(name='X'), Variable(name='Y'), Function(name='c2', arguments=[])])])), Literal(negated=True, atom=Predicate(name='better', arguments=[Function(name='f', arguments=[Variable(name='X')]), Function(name='g', arguments=[Variable(name='Y')])])), Literal(negated=False, atom=Predicate(name='$false', arguments=[])), Literal(negated=False, atom=Predicate(name='this_is_a_test_case', arguments=[]))], label='test', inference_parents=None, inference_rule=None, processed=None, birth_step=None)
+    cnf(test, hypothesis, f(X,g(Y),h(Z,c1)) = f(X,Y,c2) | ~better(f(X), g(Y)) | $false() | this_is_a_test_case()).
     """
 
     def __default_token__(self, token):
@@ -172,7 +172,19 @@ class CNFParser(Transformer):
         return Clause(literals)
 
     @staticmethod
-    def cnf_annotated(children):
+    def _parse_inference_parents(inference_parents):
+        if isinstance(inference_parents[0], list):
+            clause_inference_parents = list(
+                map(
+                    itemgetter(0),
+                    inference_parents,
+                )
+            )
+        else:
+            clause_inference_parents = [inference_parents[0]]
+        return clause_inference_parents
+
+    def cnf_annotated(self, children):
         """
         <cnf_annotated>        ::= cnf(<name>,<formula_role>,<cnf_formula> <annotations>).
 
@@ -187,10 +199,9 @@ class CNFParser(Transformer):
                         clause.inference_rule = annotation["inference_record"][
                             0
                         ]
-                        clause.inference_parents = list(
-                            map(
-                                itemgetter(0),
-                                annotation["inference_record"][1],
+                        clause.inference_parents = (
+                            self._parse_inference_parents(
+                                annotation["inference_record"][1]
                             )
                         )
         return clause
